//-------------------------------------------------------------------------------
// TMCM-AC-840 Motor Control Daughter Board Kit
// TMC603A BLDC motor commutation demonstration for
// Actel SmartFusion Development Kits (not Evaluation Kits!)
// 
// Simple SmartFusion microcontroller subsystem (MSS) example program.
// The OLED display on the SmartFusion dev kit displays a content of the
// TMC fabric block through the internal I2C interface 0
//
// The FPGA fabric is connected via the APB3.
// 
//-------------------------------------------------------------------------------
//
// Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. Redistributions 
// in binary form must reproduce the above copyright notice, this list of 
// conditions and the following disclaimer in the documentation and/or 
// other materials provided with the distribution. 
//
// Neither the name of the TRINAMIC nor the names of its contributors may 
// be used to endorse or promote products derived from this software without 
// specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
// THE POSSIBILITY OF SUCH DAMAGE.
//
//-------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include "hal.h"
#include "cortex_nvic.h"
#include "mss_gpio.h"
#include "mss_i2c.h"
#include "mss_ace.h"
#include "mss_uart.h"
#include "mss_watchdog.h"
#include "oled.h"
#include "from.h"


// fabric control register defines
#define CTRL_REG_0_ENN_TMC262_ADDR    0x400F0000UL
#define CTRL_REG_1_ENN_TMC603_ADDR    0x400F0004UL
#define CTRL_REG_2_CTRL_FOCUS_ADDR    0x400F0008UL
#define CTRL_REG_3_CTRL_SELEC_ADDR    0x400F000CUL
#define CTRL_REG_4_SAMPLEx603_ADDR    0x400F0010UL
#define CTRL_REG_5_CONFIG_SPI_ADDR    0x400F0014UL
// fabric data register defines
#define TMC603_PWM_VAL_REG_ADDR       0x400F0018UL
#define TMC262_WAIT_CNT_REG_ADDR      0x400F001CUL
#define TMC262_MICROSTEP_RES_REG_ADDR 0x400F0020UL
#define TMC262_SPEED_VAL_REG_ADDR     0x400F0024UL
// fabric control register defines
#define CTRL_REG_BIT0_MASK		 0x00000001UL
#define CTRL_REG_BIT1_MASK	     0x00000002UL
#define CTRL_REG_BIT2_MASK       0x00000004UL
#define CTRL_REG_BIT3_MASK	     0x00000008UL
#define CTRL_REG_BIT4_MASK	     0x00000010UL
#define CTRL_REG_BIT5_MASK	     0x00000020UL
#define CTRL_REG_BIT6_MASK	     0x00000040UL
#define CTRL_REG_BIT7_MASK	     0x00000080UL

// Menu option defines
#define TMC262_ENN  '0'
#define TMC603_ENN  '1'
#define CTRL_FOCUS  '2'
#define CTRL_SPEED  '3'
#define CTRL_DIREC  '4'
#define CTRL_MIRES  '5'
#define CTRL_SAMPL  '6'
#define TMC262_SPI  '7'

// variables etc...
uint32_t CTRL_REG_0_ENN_TMC262_VAL; // 0 = off, 1 = on
uint32_t CTRL_REG_1_ENN_TMC603_VAL; // 0 = off, 1 = on
uint32_t CTRL_REG_2_CTRL_FOCUS_VAL; // 0 = TMC603, 1 = TMC262
uint32_t CTRL_REG_3_CTRL_SELEC_VAL; // 0 = nothing, 1 = speed, 2 = direction, 3 = micro step resolution
uint32_t CTRL_REG_4_SAMPLEx603_VAL; // 0 = hold, 1 = sample
uint32_t CTRL_REG_5_CONFIG_SPI_VAL; // 0 = default, 1 = trigger SPI configuration

uint8_t display_buffer[20];
uint8_t key = '-';
uint8_t driver_old = 0;
uint8_t rx_size = 0;

uint32_t TMC603_pwm;
uint32_t TMC262_WC;
uint32_t TMC262_MR;
uint32_t TMC262_speed;

uint16_t adc_result;
float adc_result_cur1_mV;
float adc_result_cur2_mV;
float adc_result_cur3_mV;
float adc_result_vm603_mV;
float adc_result_vm262_mV;

// functions
void init_system();
void OLED_startup_message();
void show_main_menu();
void update_OLED(uint8_t driver);
void process_uart_data();
    
int main()
{
    // contains all basic system initialization stuff
	init_system();

	// Greeting Message on OLED at startup
	OLED_startup_message();
	
	// show menu via UART
	show_main_menu();

	// main loop
	while(1)
    { 
		// poll the UART
		rx_size = MSS_UART_get_rx( &g_mss_uart0, &key, 1);
		if( rx_size == 1 )
		{
			rx_size = 0;
			process_uart_data();
		}
		// Always update OLED
		// with check on '-' for first update after reset (so the startup message is not deleted
		if (key != '-')
		{
			update_OLED(HW_get_32bit_reg( CTRL_REG_2_CTRL_FOCUS_ADDR ));
		}
    }
}

//////////////////////////////////////////
void process_uart_data()
   {
	   switch(key)
       {
		   // reg_value |= CNTRL_REG_0_BIT11_MASK  // set to '1', set
       	   // reg_value &= ~CNTRL_REG_0_BIT8_MASK; // set to '0', clear
           // reg_value ^= CNTRL_REG_0_BIT8_MASK;  // toggle
		   case TMC262_ENN: // toggle ENN signal of TMC262
		   {
			   CTRL_REG_0_ENN_TMC262_VAL = HW_get_32bit_reg( CTRL_REG_0_ENN_TMC262_ADDR ); // get current value
			   //printf("CTRL_REG_0 read: %08X\n\r", CTRL_REG_0_ENN_TMC262_VAL);
			   CTRL_REG_0_ENN_TMC262_VAL ^= CTRL_REG_BIT0_MASK;                           // toggle the ENN bit
			   //printf("CTRL_REG_0 new : %08X", CTRL_REG_0_ENN_TMC262_VAL);
			   HW_set_32bit_reg( CTRL_REG_0_ENN_TMC262_ADDR, CTRL_REG_0_ENN_TMC262_VAL );  // write updated value
			   break;
		   }
		   case TMC603_ENN: // toggle ENN signal of TMC603A
		   {
			   CTRL_REG_1_ENN_TMC603_VAL = HW_get_32bit_reg( CTRL_REG_1_ENN_TMC603_ADDR ); // get current value
			   //printf("CTRL_REG_1 read: %08X\n\r", CTRL_REG_1_ENN_TMC603_VAL);
			   CTRL_REG_1_ENN_TMC603_VAL ^= CTRL_REG_BIT0_MASK;                            // toggle the ENN bit
			   //printf("CTRL_REG_1 new : %08X", CTRL_REG_1_ENN_TMC603_VAL);
			   HW_set_32bit_reg( CTRL_REG_1_ENN_TMC603_ADDR, CTRL_REG_1_ENN_TMC603_VAL );  // write updated value
			   break;
		   }
		   case CTRL_FOCUS: // set control mux in FPGA to 262, change OLED information to 262
		   {
			   CTRL_REG_2_CTRL_FOCUS_VAL = HW_get_32bit_reg( CTRL_REG_2_CTRL_FOCUS_ADDR ); // get current value
			   //printf("CTRL_REG_2 read: %08X\n\r", CTRL_REG_2_CTRL_FOCUS_VAL);
			   CTRL_REG_2_CTRL_FOCUS_VAL ^= CTRL_REG_BIT0_MASK;                           // toggle the bit
			   //printf("CTRL_REG_2 new : %08X", CTRL_REG_2_CTRL_FOCUS_VAL);
			   HW_set_32bit_reg( CTRL_REG_2_CTRL_FOCUS_ADDR, CTRL_REG_2_CTRL_FOCUS_VAL );  // write updated value
			   update_OLED(CTRL_REG_2_CTRL_FOCUS_VAL);                                    // update OLED disply
			   break;
		   }
		   case CTRL_SPEED: // change SW1/SW2 mux to speed control
		   {
			   //CTRL_REG_3_CTRL_SELEC_VAL = 1;                   // set to 1 = speed control
			   HW_set_32bit_reg( CTRL_REG_3_CTRL_SELEC_ADDR, 0x00000001 );  // write updated value
			   break;
		   }
		   case CTRL_DIREC: // change SW1/SW2 mux to direction control
		   {
			   //CTRL_REG_3_CTRL_SELEC_VAL = 2;                   // set to 2 = direction control
			   HW_set_32bit_reg( CTRL_REG_3_CTRL_SELEC_ADDR, 0x00000002 );  // write updated value
			   break;
		   }
		   case CTRL_MIRES: // change SW1/SW2 mux to microstep resolution control
		   {
			   //CTRL_REG_3_CTRL_SELEC_VAL = 3;                   // set to 3 = Microstep resolution control
			   HW_set_32bit_reg( CTRL_REG_3_CTRL_SELEC_ADDR, 0x00000003 );  // write updated value
			   break;
		   }
		   case CTRL_SAMPL: // toggle state of SAMPLEx
		   {
			   CTRL_REG_4_SAMPLEx603_VAL = HW_get_32bit_reg( CTRL_REG_4_SAMPLEx603_ADDR ); // get current value
			   //printf("CTRL_REG_4 read: %08X\n\r", CTRL_REG_4_SAMPLEx603_VAL);
			   CTRL_REG_4_SAMPLEx603_VAL ^= CTRL_REG_BIT0_MASK;                           // toggle the bit
			   //printf("CTRL_REG_4 read: %08X", CTRL_REG_4_SAMPLEx603_VAL);
			   HW_set_32bit_reg( CTRL_REG_4_SAMPLEx603_ADDR, CTRL_REG_4_SAMPLEx603_VAL );  // write updated value
			   break;
		   }
		   case TMC262_SPI: // trigger SPI configuration for TMC262
		   {
			   //CTRL_REG_5_CONFIG_SPI_VAL = 1;                   // set to 1 = (re)configure TMC262, e.g., when microstep resolution has changed
			   HW_set_32bit_reg( CTRL_REG_5_CONFIG_SPI_ADDR, 1 );  // write updated value
			   break;
		   }
		   default:  /* If selected key is out of range */
		   {
			   //printf("!!!! %c is an invalid selection, please retry !!!!                           ", key);
			   break;
		   }
       }
       // show menu via UART again, now with last key pressed
       show_main_menu();
   }
//////////////////////////////////////////
void init_system()
{
    // system init
    SystemInit();    
    // disable watchdog
    MSS_WD_disable();    
    // ACE init
    ACE_init();
    // GPIO init
    MSS_GPIO_init();    
    // Configure MSS GPIOs
    //Set OLED DC# to GND, otherwise it might not work
    MSS_GPIO_config( MSS_GPIO_19, MSS_GPIO_OUTPUT_MODE );
    MSS_GPIO_set_output(MSS_GPIO_19, 0);
    
    OLED_init();
    OLED_clear_display();
    // UART init
    MSS_UART_init( &g_mss_uart0, MSS_UART_57600_BAUD, MSS_UART_DATA_8_BITS | MSS_UART_NO_PARITY | MSS_UART_ONE_STOP_BIT );

}

/////////////////////////////////////////
void OLED_startup_message()
{
	OLED_clear_display();
	OLED_set_cursor( FIRST_LINE,FIRST_CHARACTER );
	OLED_write_string("Hi, welcome to TMC!");
	OLED_set_cursor( SECOND_LINE,FIRST_CHARACTER );
	OLED_write_string("Motor Control Demo ");	
}
///////////////////////////////////////////
void show_main_menu()
{
	printf( "\n\r" );
	printf( "**********************************************************************\n\r" );
	printf( "****  Welcome to Trinamic's Embedded Motor Control Demonstration  ****\n\r" );
	printf( "****             for Actel SmartFusion Dev & Eval Kits            ****\n\r" );
	printf( "**********************************************************************\n\r" );
	printf( "**** Choose an Option!                                            ****\n\r" );  // TO DO
	printf( "****                                                              ****\n\r" );
	printf( "**** 0. Toggle ENABLE of TMC262 stepper driver MOSFET bridges     ****\n\r" );  // toggle ENN signal of TMC262
	printf( "**** 1. Toggle ENABLE of TMC603A BLDC driver MOSFET bridges       ****\n\r" );  // toggle ENN signal of TMC603A
	printf( "**** 2. Toggle control focus (stepper or BLDC motor)              ****\n\r" );  // toggle control mux in FPGA, change OLED information, default = non
	printf( "**** 3. Speed Control using SW1/SW2                               ****\n\r" );  // change SW1/SW2 mux to speed control
	printf( "**** 4. Direction Control using SW1/SW2                           ****\n\r" );  // change SW1/SW2 mux to direction control
    printf( "**** 5. Change Microstep Resolution using SW1/SW2 (TMC262 only)   ****\n\r" );  // change SW1/SW2 mux to MR control
	printf( "**** 6. Toggle SAMPLEx signals of TMC603A                         ****\n\r" );  // toggle state of SAMPLEx
	printf( "**** 7. (Re)Configure TMC262 with latest configuration settings   ****\n\r" );  // trigger SPI config for TMC262
	printf( "****                                                              ****\n\r" );
	printf( "**********************************************************************\n\r" );
	printf( "**** For status information watch the OLED!                       ****\n\r" );
	printf( "**********************************************************************\n\r" );
	printf( "**** Your last input was: %c                                       ****\n\r", key );
	printf( "**** (only inputs 0-7 are allowed)                                ****\n\r" );
	printf( "**********************************************************************\n\r" );
	printf( "\n\r" );
	printf( "\n\r" );
}
///////////////////////////////////////////
void update_OLED(uint8_t driver)
{	

	if (driver == 0) {
		if (driver != driver_old)
		{
			OLED_clear_display();
			driver_old = driver;
		}
		// set and format the first OLED line 
		OLED_set_cursor( FIRST_LINE,FIRST_CHARACTER );
			  
		// TMC603A PWM value and Supply voltage of TMC603A block
		TMC603_pwm = HW_get_32bit_reg( TMC603_PWM_VAL_REG_ADDR );
		adc_result = ACE_get_ppe_sample( VM603_FPGA );
		adc_result_vm603_mV = 21 * ACE_convert_to_mV (VM603_FPGA, adc_result);
		snprintf((char *) display_buffer, sizeof(display_buffer), "603: PWM=x%X U%.1fV ",
				                                                  TMC603_pwm,
				                                                  ((float)(adc_result_vm603_mV) / (float)(1000)) );     
		OLED_write_string((char *) display_buffer);
			  
		// set and format the second OLED line
		OLED_set_cursor( SECOND_LINE,FIRST_CHARACTER );
			  
		// CURx outputs of TMC603A
		adc_result = ACE_get_ppe_sample( CUR1_603 );
		adc_result_cur1_mV = ACE_convert_to_mV (CUR1_603, adc_result);		  
		adc_result = ACE_get_ppe_sample( CUR2_603 );
		adc_result_cur2_mV = ACE_convert_to_mV (CUR2_603, adc_result);	
		adc_result = ACE_get_ppe_sample( CUR3_603 );
		adc_result_cur3_mV = ACE_convert_to_mV (CUR3_603, adc_result);
		snprintf((char *) display_buffer, sizeof(display_buffer), "%.2fV %.2fV %.2fV ", ((float)(adc_result_cur1_mV) / (float)(1000)),
				                                                                        ((float)(adc_result_cur2_mV) / (float)(1000)),
				                                                                        ((float)(adc_result_cur3_mV) / (float)(1000)));	
		OLED_write_string((char *)display_buffer);		  
	
	} else {
		if (driver != driver_old)
		{
			OLED_clear_display();
			driver_old = driver;
		}
		
		// set and format the first OLED line
		OLED_set_cursor( FIRST_LINE,FIRST_CHARACTER );
		
		TMC262_WC = HW_get_32bit_reg( TMC262_WAIT_CNT_REG_ADDR );
		TMC262_MR = HW_get_32bit_reg( TMC262_MICROSTEP_RES_REG_ADDR );
		TMC262_speed = HW_get_32bit_reg( TMC262_SPEED_VAL_REG_ADDR );
						
		snprintf((char *) display_buffer, sizeof(display_buffer), "262: Vx%X MR%d ", TMC262_speed, TMC262_MR );      
		OLED_write_string((char *) display_buffer);		  
		
		// set and format the first OLED line
		OLED_set_cursor( SECOND_LINE,FIRST_CHARACTER );
	          
		// actual wait cycles (= # of clock cycles between two step impulses at the step/dir interface) and Supply Voltage of TMC262 block
		adc_result = ACE_get_ppe_sample( VM262_FPGA );
		adc_result_vm262_mV = 21 * ACE_convert_adc_input_to_mV (VM262_FPGA, adc_result);
		snprintf((char *) display_buffer, sizeof(display_buffer), "WCx%X U%.1fV ", (int)TMC262_WC, ((float)(adc_result_vm262_mV) / (float)(1000)) );
		OLED_write_string((char *) display_buffer);			 		  	  
		
	}
}
