----------------------------------------------------------------------------------
--                                                                              --
-- File       : PWMClockSync.vhd                                                --
--                                                                              --
-- History     : BD, September 10, 2007, Creation                               --
--               BD,  November 20, 2007, ChopDivNom expanded by 2 bit           --
--               BD,   January 18, 2008, StepTimeout implemented, to allow      --
--                                       change of chopper frequency when       --
--                                       motor is stopped                       --
--               BD,  September 2, 2008, changed RESET to NRESET, removed       --
--                                       STEPPULSE for appnote                  --
--               SL,   October 23, 2008, Translation of comments to english     --
--                                                                              --
----------------------------------------------------------------------------------
--                                                                              --
-- Hints : CHOPDIVNOM can be calculated by dividing the clock frequency by      --
--         the nominal chopper clock frequency, e.g. for 36kHz chopper clock    --
--         frequency and 16MHz clock grequency, CHOPDIVNOM is                   --
--         16000000/36000 = 444 (rounded) or "000110111100" as std_logic_vector --
--				 The value must be smaller than 818             --
--                                                                              --
--         The frequency is slightly modulated to achieve synchronization       --
--         with the full step frequency.                                        --
--                                                                              --
----------------------------------------------------------------------------------
-- Copyright (c) 2008, TRINAMIC Motion Control GmbH & Co. KG                    --
-- All rights reserved.                                                         --
--                                                                              --
-- Redistribution and use in source and binary forms, with or without           --
-- modification, are permitted provided that the following conditions are met:  --
--                                                                              --
-- * Redistributions of source code must retain the above copyright notice,     --
--   this list of conditions and the following disclaimer.                      --
--                                                                              --
-- * Redistributions in binary form must reproduce the above copyright notice,  --
--   this list of conditions and the following disclaimer in the documentation  --
--   and/or other materials provided with the distribution.                     --
--                                                                              --
-- * Neither the name TRINAMIC nor the names of its contributors may be used    --
--   to endorse or promote products derived from this software without          --
--   specific prior written permission.                                         --
--                                                                              --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  --
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    --
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   --
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE     --
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR          --
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         --
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS     --
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      --
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)      --
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE   --
-- POSSIBILITY OF SUCH DAMAGE.                                                  --
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity PWMClockSync is
  
  port (
    CLK           : in  std_logic;
    NRESET        : in  std_logic;
    STEPPHASE     : in  std_logic;
    CHOPDIVNOM    : in  std_logic_vector (11 downto 0);
    CHOPPULSE_OUT : out std_logic;
    CHOPCLK_SYM   : out std_logic
    );

end PWMClockSync;

architecture structure of PWMClockSync is

  signal StepPhase_alt_ff     : std_logic;
  signal Anzahl_ff, AnzCnt_ff : std_logic_vector(7 downto 0);
  signal Rest_ff              : std_logic_vector(12 downto 0);
  signal RestCnt_ff           : std_logic_vector(11 downto 0);
  signal TrigChop_ff          : std_logic;
  signal StartMess_ff         : std_logic;
  signal Chop_ff, ChopSym_ff  : std_logic;
  signal ChopperCnt_ff        : std_logic_vector(11 downto 0);
  signal ChopperLimit_ff      : std_logic_vector(11 downto 0);
  signal StepTimeout_ff       : std_logic;

begin

  CHOPPULSE_OUT <= Chop_ff;
  CHOPCLK_SYM   <= ChopSym_ff;

  Counter_p : process (CLK, NRESET)
  begin
    if NRESET = '0' then

      StepPhase_alt_ff <= '0';
      RestCnt_ff       <= "000000000000";
      AnzCnt_ff        <= "00000000";
      TrigChop_ff      <= '0'; StartMess_ff <= '1';
      StepTimeout_ff   <= '0';
      
    elsif CLK'event and CLK = '1' then

      TrigChop_ff    <= '0';
      StepTimeout_ff <= '0';
      if StartMess_ff = '1' then
        StartMess_ff <= '0';
        RestCnt_ff   <= "000000000001";
        AnzCnt_ff    <= "00000000";
      else
        RestCnt_ff <= RestCnt_ff + 1;
        if (RestCnt_ff = CHOPDIVNOM) then
          RestCnt_ff  <= "000000000000";
          if AnzCnt_ff /= "11111111" then
            AnzCnt_ff <= AnzCnt_ff + 1;
          else
            StepTimeout_ff <= '1';
          end if;
        end if;

        if (STEPPHASE /= StepPhase_alt_ff) then  -- New full step
          StepPhase_alt_ff <= STEPPHASE;
          StartMess_ff     <= '1';
          if (AnzCnt_ff /= "11111111") and (AnzCnt_ff > 3) then  -- start only if full step frequency is not too low or too high
            TrigChop_ff    <= '1';         -- start new chopper cycle
          end if;
        end if;
      end if;
    end if;
  end process Counter_p;

  Chopper_p : process (CLK, NRESET)
  begin
    if NRESET = '0' then

      Rest_ff         <= "0000000000000";
      Anzahl_ff       <= "00000000";
      ChopperCnt_ff   <= "000000000000";
      ChopperLimit_ff <= "000000000000";
      Chop_ff         <= '0';
      ChopSym_ff      <= '0';
      
    elsif CLK'event and CLK = '1' then

      -- Chopper for TMC236/TMC239/TMC246/TMC249
      if ChopperCnt_ff > ('0' & ChopperLimit_ff(11 downto 1)) then
        ChopSym_ff <= '1';
      else
        ChopSym_ff <= '0';
      end if;

      if StartMess_ff = '1' or StepTimeout_ff = '1' then
        ChopperLimit_ff <= CHOPDIVNOM;
      end if;
      if TrigChop_ff = '1' then
        Anzahl_ff <= AnzCnt_ff;
        Rest_ff   <= ('0' & RestCnt_ff) + ("000" & CHOPDIVNOM(11 downto 2));
      else
        if (Rest_ff /= "0000000000000") and (Rest_ff >= ('0' & Anzahl_ff)) then
          Rest_ff         <= Rest_ff - ('0' & Anzahl_ff);
          ChopperLimit_ff <= ChopperLimit_ff + 1;
        end if;
      end if;

      -- Choppertimer
      Chop_ff       <= '0';
      ChopperCnt_ff <= ChopperCnt_ff + 1;
      if (ChopperCnt_ff >= ChopperLimit_ff) or (TrigChop_ff = '1') then
        ChopperCnt_ff <= "000000000001";  -- start new chopper cycle
        Chop_ff       <= '1';
      end if;
    end if;
    
  end process Chopper_p;
end structure;
