-------------------------------------------------------------------------------
--                                                                           --
-- File         : bridge_APB3_to_userlogic.vhd                               --
--                                                                           --
-- Author(s)    : Dr. Stephan Kubisch (SK), www.trinamic.com                 --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : simple macro module to connect user logic to APB3 slave 
--
-- Creation     : SK, March 08, 2010                                         --
--                                                                           --
-- History      : SK, March 08, 2010 - Basic Structure, Signals, Ports       --
--                                     different generates for APB           --
--                SK, March 09, 2010 - simple APB transfers implemented      --
--                                     generic bus addr width
--                                                                           --
-------------------------------------------------------------------------------
--                                                                           --
-- Hint(s) : Similiar to the AMBA specification, the naming convention and
-- meaning of this module is from the bus perspective. That is, a write
-- transfer is a data transfer from the APB bus interface to the user
-- logic, e.g., a microprocessor writes to a peripheral. A read transfer is a
-- data transfer from the user logic to the bus, e.g., sending data from an
-- FPGA block to a microprocessor.
--                                                                           --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.pkg_commutator_components.all;

entity bridge_APB3_to_userlogic is
  
  port (
    -- APB3 interface signals to/from APB3 slave port
    PCLK              : in  std_logic;
    PRESETn           : in  std_logic;  -- active low!
    PENABLE           : in  std_logic;
    PSEL              : in  std_logic;
    PADDR             : in  std_logic_vector(23 downto 0);
    PWRITE            : in  std_logic;
    PWDATA            : in  std_logic_vector(31 downto 0);
    PRDATA            : out std_logic_vector(31 downto 0);
    PREADY            : out std_logic;  -- slave uses PREADY to extent a transfer with wait cycles
    PSLVERR           : out std_logic;
    -- interface to user logic
    U_WR_REQ          : out std_logic;  -- notify a write from bus to userlogic
    U_WR_ACK          : in  std_logic;  -- acknowledge that user side has received the data (U_WR_DATA)
    U_WR_DATA         : out std_logic_vector(31 downto 0);
    U_WR_ADDR         : out std_logic_vector(23 downto 0);
    CTRL_REG_0        : in  std_logic_vector(31 downto 0);
    CTRL_REG_1        : in  std_logic_vector(31 downto 0);
    CTRL_REG_2        : in  std_logic_vector(31 downto 0);
    CTRL_REG_3        : in  std_logic_vector(31 downto 0);
    CTRL_REG_4        : in  std_logic_vector(31 downto 0);
    CTRL_REG_5        : in  std_logic_vector(31 downto 0);
    U_RD_DATA_603_PWM : in  std_logic_vector(31 downto 0);
    U_RD_DATA_262_WC  : in  std_logic_vector(31 downto 0);
    U_RD_DATA_262_MR  : in  std_logic_vector(31 downto 0);
    U_RD_DATA_262_SV  : in  std_logic_vector(31 downto 0)
    );

end bridge_APB3_to_userlogic;


architecture arch of bridge_APB3_to_userlogic is

  type apb_wsm_state_type is (idle_and_address,   -- states for apb write state machine
                              data_phase);
  type apb_rsm_state_type is (idle_and_address,   -- states for apb read state machine
                              data_phase);

  type reg_type_APB is record           -- global registers
    -- state machine registers
    apb_wsm_state : apb_wsm_state_type;
    apb_rsm_state : apb_rsm_state_type;
    -- APB3 bus interface registered outputs
    prdata_ff     : std_logic_vector(31 downto 0);
    pready_ff     : std_logic;
    pslverr_ff    : std_logic;
    -- user side registered outputs
    u_wr_data_ff  : std_logic_vector(31 downto 0);
    u_wr_addr_ff  : std_logic_vector(23 downto 0);
    u_wr_req_ff   : std_logic;
  end record;
  
  signal r_APB, rin_APB : reg_type_APB;             -- global register signals
  signal CLK, nRST : std_logic;                     -- active low

begin  -- arch

    CLK   <= PCLK;                      -- use this as clock here
    nRST  <= PRESETn;                   -- use this as reset here
    
    -- update/reset register values
    regs : process (CLK, nRST)
    begin  -- process regs
      if nRST = '0' then                 -- asynchronous reset (active low)
        r_APB.apb_wsm_state <= idle_and_address;
        r_APB.apb_rsm_state <= idle_and_address;
        r_APB.prdata_ff     <= (others => '0');
        r_APB.pready_ff     <= '0';
        r_APB.u_wr_data_ff  <= (others => '0');
        r_APB.u_wr_req_ff   <= '0';
      elsif rising_edge(CLK) then
        r_APB <= rin_APB;
      end if;
    end process regs;

    -- functionality
    functionality : process (r_APB, PENABLE, PSEL, PADDR, PWRITE, PWDATA,
                             U_RD_DATA_603_PWM, U_RD_DATA_262_WC, U_RD_DATA_262_MR, U_RD_DATA_262_SV,
                             CTRL_REG_0, CTRL_REG_1, CTRL_REG_2, CTRL_REG_3, CTRL_REG_4, CTRL_REG_5 )
      variable v : reg_type_APB;
    begin  -- process functionality
      
      -- default assignments
      v            := r_APB;             -- get current register values
      v.pready_ff  := '1';               -- tied high
      v.pslverr_ff := '0';               -- tied low
      
      -- APB3 Write State Machine, currently, just simple transfers (1 Data-Word) are implemented
      -- The bus "writes" data to the connected slave, e.g, a user FPGA block.
      -- The PREADY signal can extent a transfer with wait cyles, here it is
      -- always tied high for typical 2 cycle transfers.
      case r_APB.apb_wsm_state is
        when idle_and_address =>
          -- if data is available check addr and forward to user logic
          if (PSEL = '1' and PWRITE = '1' and PENABLE = '0') then
            v.apb_wsm_state := data_phase;  -- change state in any (address) case
            -- modify this condition if more slave addresses are needed
            v.u_wr_data_ff := PWDATA;
            v.u_wr_addr_ff := PADDR; 
            v.u_wr_req_ff  := '1';      -- notify user logic that new data is available
          else
            v.u_wr_data_ff := (others => '0');
            v.u_wr_addr_ff := (others => '0');
            v.apb_wsm_state := idle_and_address;    -- remain here
          end if;
        when data_phase =>
          -- nothing to do here
          -- PREADY is always high (see assignment above)
          -- needs to be modified if wait cycles are used.
          v.u_wr_req_ff  := '0';
          v.apb_wsm_state := idle_and_address;    -- remain here
        when others =>
          v.apb_wsm_state := idle_and_address;  -- default state
          v.u_wr_req_ff   := '0';
          v.u_wr_addr_ff  := (others => '0');
          v.u_wr_data_ff  := (others => '0');
      end case;

      -- reset the user notification for new data when user logic acknowledges reception
      -- ! The user logic is responsible for sampling the new data, since 
      --   it will be overwritten when new data arrives from the bus !
      if r_APB.u_wr_req_ff = '1' and U_WR_ACK = '1' then
        v.u_wr_req_ff := '0';
      end if;

        
      -- APB3 Read State Machine
      -- The slave writes data to the bus, e.g., user FPGA logic is asked to send parameters to the MSS
      case r_APB.apb_rsm_state is
        when idle_and_address =>
          -- if user logic wants to write data to the bus
          if (PSEL = '1' and PWRITE = '0' and PENABLE = '0') then
            -- modify addresses and data at your needs
            case PADDR is

              when TMC603_PWM_VAL_REG_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := U_RD_DATA_603_PWM; -- assign NEW slave data
              when TMC262_WC_REG_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := U_RD_DATA_262_WC;  -- assign NEW slave data
              when TMC262_MR_REG_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := U_RD_DATA_262_MR;  -- assign NEW slave data
              when TMC262_SPEED_VAL_REG_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := U_RD_DATA_262_SV;  -- assign NEW slave data
              when CTRL_REG_0_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := CTRL_REG_0;        -- assign NEW slave data
              when CTRL_REG_1_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := CTRL_REG_1;        -- assign NEW slave data
              when CTRL_REG_2_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := CTRL_REG_2;        -- assign NEW slave data
              when CTRL_REG_3_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := CTRL_REG_3;        -- assign NEW slave data
              when CTRL_REG_4_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := CTRL_REG_4;        -- assign NEW slave data
              when CTRL_REG_5_ADDR =>  
                v.apb_rsm_state := data_phase;        -- change state
                v.prdata_ff     := CTRL_REG_5;        -- assign NEW slave data
              when others =>
                v.apb_rsm_state := idle_and_address;  -- remain here
                v.prdata_ff     := (others => '0');
            end case;
          else
            v.apb_rsm_state := idle_and_address;
            v.prdata_ff     := (others => '0');
          end if;
        when data_phase =>
          v.apb_rsm_state := idle_and_address;
          v.prdata_ff     := (others => '0');
        when others =>
          v.apb_rsm_state := idle_and_address;
          v.prdata_ff     := (others => '0');
      end case;

    -- Assign new register values
    rin_APB   <= v;
    -- drive outputs, use registered outputs (r_APB)
    PRDATA    <= r_APB.prdata_ff;
    PREADY    <= r_APB.pready_ff;
    PSLVERR   <= r_APB.pslverr_ff;
    U_WR_DATA <= r_APB.u_wr_data_ff;
    U_WR_ADDR <= r_APB.u_wr_addr_ff;
    U_WR_REQ  <= r_APB.u_wr_req_ff;

    end process functionality;
  
end arch;
