-------------------------------------------------------------------------------
--                                                                           --
-- File         : chopcntl_uvw.vhd                                           --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description : TMC60x commutator 'chopcntl_uvw' block (from schematic)     --
--                                                                           --
--               Brake        : active brake signal (switches LowSide on and --
--                              High Side off, PWM choppers the brake)       --
--               DualChop     : Chopper HS and LS / Chopper Single Side      --
--               ChopHS       : Chopper High Side only / Low Side only       --
--               PWM_and_nOVC : PWM signal input (anded with NOT OverCurrent)--
--                                                                           --
--               H_IN / L_IN  : Hi Side / Low Side                           --
--                                                                           --
--               H_GATE       : Gate Control Signal of HiSide FETs           --
--               L_GATE       : Gate Control Signal of LowSide FETs          --
--                                                                           --
-- Creation     : LL, July 24, 2008                                          --
--                                                                           --
-- History      : LL, February 6, 2010 : BSD licensing scheme added          --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity chopcntl_uvw is 
  port ( 
         DualChop       :  in std_Logic;
         ChopHS         :  in std_Logic;
         
         PWMnOVC        :  in std_logic;
         
         H_IN_W, L_IN_W :  in std_logic;
         H_W,    L_W    : out std_logic;

         H_IN_V, L_IN_V :  in std_logic;
         H_V,    L_V    : out std_logic;

         H_IN_U, L_IN_U :  in std_logic;
         H_U,    L_U    : out std_logic);
  
end chopcntl_uvw;

architecture behavior of chopcntl_uvw is

  signal dcp        : std_logic_vector ( 2 downto 0 ); 

  signal dcp_h_in_w : std_logic_vector ( 3 downto 0 ); 
  signal dcp_l_in_w : std_logic_vector ( 3 downto 0 ); 
           
  signal dcp_h_in_v : std_logic_vector ( 3 downto 0 ); 
  signal dcp_l_in_v : std_logic_vector ( 3 downto 0 ); 
             
  signal dcp_h_in_u : std_logic_vector ( 3 downto 0 ); 
  signal dcp_l_in_u : std_logic_vector ( 3 downto 0 ); 

begin


  dcp <= DualChop & ChopHS & PWMnOVC;      
  
  dcp_h_in_w <= dcp & H_IN_W; 
  dcp_l_in_w <= dcp & L_IN_W;
                            
  dcp_h_in_v <= dcp & H_IN_V;
  dcp_l_in_v <= dcp & L_IN_V;
                        
  dcp_h_in_u <= dcp & H_IN_U;
  dcp_l_in_u <= dcp & L_IN_U;
  
  
  with dcp_h_in_w select H_W <= '1' when "0001", '1' when "0011", '1' when "0111", '1' when "1011", '1' when "1111", '0' when others; 
  with dcp_l_in_w select L_W <= '1' when "0011", '1' when "0101", '1' when "0111", '1' when "1011", '1' when "1111", '0' when others;                                                                                                                            
                                   
  with dcp_h_in_v select H_V <= '1' when "0001", '1' when "0011", '1' when "0111", '1' when "1011", '1' when "1111", '0' when others;                                                                                                                            
  with dcp_l_in_v select L_V <= '1' when "0011", '1' when "0101", '1' when "0111", '1' when "1011", '1' when "1111", '0' when others;   
                                   
  with dcp_h_in_u select H_U <= '1' when "0001", '1' when "0011", '1' when "0111", '1' when "1011", '1' when "1111", '0' when others; 
  with dcp_l_in_u select L_U <= '1' when "0011", '1' when "0101", '1' when "0111", '1' when "1011", '1' when "1111", '0' when others; 

end behavior; 

---------------------------------------------------------------------------------------
