-------------------------------------------------------------------------------
--                                                                           --
-- File         : hall_chk.vhd                                               --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : checks commutation (Hall signal sequence) and generates    --
--                tacho signal and direction signal, polarity of direction   --
--                on comutation fault detection                              --
--                                                                           --
-- Creation     : LL,   March 19, 2009                                       --
--                                                                           --
-- History      : LL, February 6, 2010 : BSD license added                   --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      :                                                            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity hall_chk is
  port (
    nRST     : in  std_logic;           -- high active reset input
    CLK      : in  std_logic;           -- clock input
    H1       : in  std_logic;           -- hall sensor signal H1
    H2       : in  std_logic;           -- hall sensor signal H2
    H3       : in  std_logic;           -- hall sensor signal H3       
    STEP_OUT : out std_logic;  -- one step pulse for each commutation step (tacho signal)
    DIR_OUT  : out std_logic  -- direction output ('0':forward / '1':backward), 
   -- direction toggles on commutation errors (000 or 111 vector)
    );
end hall_chk;

architecture behavior of hall_chk is

  signal h321_last_ff, h321_ff                  : std_logic_vector (2 downto 0);
  signal dir_ff, bw_ff, fw_ff, wrong_ff, stp_ff : std_logic;

begin


  fwbw_p :  -- determines direction of commutation : forward (fw) / backward (bw)  
  process (nRST, CLK)
  begin
    if nRST = '0' then
      h321_last_ff <= (others => '0');
      h321_ff      <= (others => '0');
      bw_ff        <= '0';
      fw_ff        <= '0';
    elsif (CLK'event and CLK = '1') then
      h321_ff <= H3 & H2 & H1; h321_last_ff <= h321_ff;
      if (                              -- backward 001 011 010 110 100 101
        ((h321_ff & h321_last_ff) = ("011" & "001")) or
        ((h321_ff & h321_last_ff) = ("010" & "011")) or
        ((h321_ff & h321_last_ff) = ("110" & "010")) or
        ((h321_ff & h321_last_ff) = ("100" & "110")) or
        ((h321_ff & h321_last_ff) = ("101" & "100")) or
        ((h321_ff & h321_last_ff) = ("001" & "101"))
        ) then bw_ff <= '1'; else bw_ff <= '0';
                             end if;
      if (                              --  forward 001 101 100 110 010 011
        ((h321_ff & h321_last_ff) = ("011" & "010")) or
        ((h321_ff & h321_last_ff) = ("010" & "110")) or
        ((h321_ff & h321_last_ff) = ("110" & "100")) or
        ((h321_ff & h321_last_ff) = ("100" & "101")) or
        ((h321_ff & h321_last_ff) = ("101" & "001")) or
        ((h321_ff & h321_last_ff) = ("001" & "011"))
        ) then fw_ff <= '1'; else fw_ff <= '0';
                             end if;
    end if;
  end process;


  wrong_p :  -- checks for commutation error (wrong hall signal vector 000 or 111)
  process (nRST, CLK)
  begin
    if nRST = '0' then
      wrong_ff <= '0';
    elsif (CLK'event and CLK = '1') then
      if ((h321_ff = "000") or ((h321_ff = "111"))) then
        wrong_ff <= '1';
      else
        wrong_ff <= '0';
      end if;
    end if;
  end process;


  dir_p :  -- direction output process (wrong_ff synchonously process (TMC60x XOR @ DIR_OUT))
  process (nRST, CLK)
  begin
    if nRST = '0' then
      dir_ff <= '0';
    elsif (CLK'event and CLK = '1') then
      if (bw_ff = '1') and (fw_ff = '0') then  -- (dir_ff is RS FF @ TMC60x)
        dir_ff <= '1' xor wrong_ff;
      elsif (bw_ff = '0') and (fw_ff = '1') then
        dir_ff <= '0' xor wrong_ff;
      end if;
    end if;
  end process;

  DIR_OUT <= wrong_ff xor dir_ff;
  --DIR_OUT <= dir_ff;

  stp_p :                               -- step output process 
  process (nRST, CLK)
  begin
    if nRST = '0' then
      stp_ff <= '0';
    elsif (CLK'event and CLK = '1') then
      if (bw_ff = '1') or (fw_ff = '1') then  -- (bw or fw drive CLK input of toggle (D) FF)
        stp_ff <= not stp_ff;
      end if;
    end if;
  end process;

  STEP_OUT <= stp_ff;

end behavior;
