--------------------------------------------------------------------------------
--
-- File        : spi_txrx_TMC262.vhd
--
-- Author      : Bernhard Dwersteg (BD)
-- Modifier(s) : Dr. Stephan Kubisch (SK)
--
-- Description : simple 20-bit SPI interface to TMC262
--
-- Hint(s)     : SPI clock rate is 1/2 f(CLK).
--
-- Warning     :
--
-- Creation    : BD, Oktober, 8th, 2007
--
-- History     : SK, May 05, 2010 - complete renewal for use with TMC262
--               SK, May 11, 2010 - bugfixing clock/CSN synchro
--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity spi_txrx_TMC262 is
  
  port (
    nRST      : in  std_logic;
    CLK       : in  std_logic;
    SEND_DATA : in  std_logic_vector(19 downto 0);  -- SPI Output to Slave, unsigned
    RECV_DATA : out std_logic_vector(19 downto 0);  -- SPI Input from Slave, unsigned
    READY     : out boolean;            -- indicates availability of SPI module
    START     : in  std_logic;          -- control to start transmission
    SCK       : out std_logic;          -- SPI serial clock generated here
    CSN       : out std_logic;          -- slave/chip select
    SDI       : out std_logic;          -- send line (to slave)
    SDO       : in  std_logic           -- receive line (from slave)
    );

end spi_txrx_TMC262;


architecture behavior of spi_txrx_TMC262 is

  signal bitcnt_ff  : std_logic_vector(4 downto 0);
  signal sendreg_ff : std_logic_vector(19 downto 0);  -- output register
  signal recvreg_ff : std_logic_vector(19 downto 0);  -- input register
  signal SDO_ff     : std_logic;
  signal SDI_ff     : std_logic;
  signal CSN_ff     : std_logic;
  signal SCK_ff     : std_logic;
  signal READY_ff   : boolean;

begin

  SCK       <= SCK_ff;
  CSN       <= CSN_ff;
  SDI       <= SDI_ff;
  RECV_DATA <= recvreg_ff;
  READY     <= READY_ff;

  spi_txrx_p : process(CLK, nRST)

  begin
    if nRST = '0' then

      bitcnt_ff  <= "00000";
      SCK_ff     <= '0';
      CSN_ff     <= '1';
      SDI_ff     <= '0';
      SDO_ff     <= '0';
      sendreg_ff <= (others => '0');
      recvreg_ff <= (others => '0');
      READY_ff   <= false;

    elsif rising_edge(CLK) then
      SDO_ff <= SDO;
      if (bitcnt_ff = "00000" and START = '1') then
        SCK_ff     <= '1';
        CSN_ff     <= '0';
        SDI_ff     <= sendreg_ff(19);
        sendreg_ff <= SEND_DATA;
        bitcnt_ff  <= "00001";
        READY_ff   <= false;
      elsif bitcnt_ff /= "00000" then
        
        if SCK_ff = '1' then            -- Data Update @ falling edge of SCK
          SDI_ff     <= sendreg_ff(19);
          sendreg_ff <= sendreg_ff(18 downto 0) & SDO;  -- shift & capture SDO at the same time
          if bitcnt_ff = "10101" then   -- Last bit already sent?
            CSN_ff     <= '1';
            bitcnt_ff  <= "00000";
            recvreg_ff <= sendreg_ff(18 downto 0) & SDO;  -- copy received bits and remaining SDO
            READY_ff   <= true;
          else
            bitcnt_ff <= bitcnt_ff + 1;
            READY_ff  <= false;
          end if;
        end if;  -- SCK_ff = '1'
        
        -- toggle SCK
        if bitcnt_ff < "10101" then 
          SCK_ff <= not SCK_ff;
        else
          SCK_ff <= '1';
        end if;
        
      else
        SCK_ff     <= '1';
        CSN_ff     <= '1';
        SDI_ff     <= '0';
        sendreg_ff <= SEND_DATA;
        bitcnt_ff  <= "00000";
        READY_ff   <= true;
      end if;  -- cnt = 0000 start = 1

    end if;  -- CLK'event
  end process;


end behavior;
