-------------------------------------------------------------------------------
--                                                                           --
-- File         : tmc603_commutator.vhd                                      --
--                                                                           --
-- Author(s)    : Dr. Lars Larsson (LL), www.trinamic.com                    --
--                                                                           --
-- Contributors :                                                            --
--                                                                           --
-- Description  : Commutation logic block for the TMC603 gate driver.        --
--                                                                           --
-- Creation     : LL, February 6, 2010                                       --
--                                                                           --
-- History      :                                                            --
--                                                                           --
-------------------------------------------------------------------------------
-- Hint(s)      : Switch Cap Filter (SCF) clock needs to be adjusted for     --
--                sensorless commutation when using TMC603 hallFX            --
-------------------------------------------------------------------------------
--
-- Copyright (c) 2010, TRINAMIC Motion Control GmbH & Co. KG 
-- All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without 
-- modification, are permitted provided that the following conditions 
-- are met:
--
-- Redistributions of source code must retain the above copyright notice, 
-- this list of conditions and the following disclaimer. Redistributions 
-- in binary form must reproduce the above copyright notice, this list of 
-- conditions and the following disclaimer in the documentation and/or 
-- other materials provided with the distribution. 
--
-- Neither the name of the TRINAMIC nor the names of its contributors may 
-- be used to endorse or promote products derived from this software without 
-- specific prior written permission. 
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-- ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGE.
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity tmc603_commutator is
  port (
    nRST              : in  std_logic;  -- high active asynchronous reset 
    CLK               : in  std_logic;  -- clock input                           
    HallFX_EN         : in  std_logic;  -- HallFX(TM) enable select, '0' : H321 / '1' : E312                           
    DualChop          : in  std_logic;  -- '0' : chopper @ HS xor LS / '1':chopper @ HS and LS
    ChopHS            : in  std_logic;  -- DualChop='1' => ChopHS='0' : chopper @ LS / ChopHS='1' : chopper @ HS
                                        -- hallFX needs DualChop='1' and ChopHS='0'                            
    OVC               : in  std_logic;  -- over current ('1':disable all bridges (can be handled by TMC603)
    PWM               : in  std_logic;  -- PWM input (e.g. from uC) to adjust speed via duty cycle                           
    DIR               : in  std_logic;  -- select direction of revolution / commutation                           
    BRAKE             : in  std_logic;  -- active brake (with PWM control)                           
    TACHO             : out std_logic;  -- tacho signal output (H3 xor H2 xor H1 resp. E3 xor E2 xor E1 )
    DIR_COM           : out std_logic;  -- measured commutation direction output (check for Hall / HallFX)
                                        -- '0' vs. '1' needs to be associated to 'cw' vs. 'ccw' rotation                              
    TMC603_BBM_EN     : out std_logic;  -- enables internal break-before-make circuitry
    TMC603_ENABLE     : out std_logic;  -- '0' : drivers disabled / '1' : bridges enabled                           
    TMC603_BH1        : out std_logic;  -- high side gate control, bridge 1 (high active)
    TMC603_BH2        : out std_logic;  -- high side gate control, bridge 2 (high active)
    TMC603_BH3        : out std_logic;  -- high side gate control, bridge 3 (high active)                               
    TMC603_BL1        : out std_logic;  -- low side gate control, bridge 1 (high active)
    TMC603_BL2        : out std_logic;  -- low side gate control, bridge 2 (high active)
    TMC603_BL3        : out std_logic;  -- low side gate control, bridge 3 (high active)                           
    TMC603_SAMPLE_123 : out std_logic;  -- common sample trigger signal for TMC603 inputs SAMPLE1, SAMPLE2, SAMPLE3                           
    TMC603_SCCLK      : out std_logic;  -- switched capacitor filter clock signal 
    TMC603_CLR_ERR    : out std_logic;  -- clear error condition flag
    TMC603_ERROR_OUT  : in  std_logic;  -- error output of TMC603 (can be dicertly connected to CLR_ERR
    TMC603_SENSE_HI   : out std_logic;  -- '0':low sense amplification / '1':high sense amplification
    TMC603_H1         : in  std_logic;  -- hallXF output of Hall Sensor emulation signal H1
    TMC603_H2         : in  std_logic;  -- hallXF output of Hall Sensor emulation signal H2
    TMC603_H3         : in  std_logic;  -- hallXF output of Hall Sensor emulation signal H3                           
    HALL1             : in  std_logic;  -- Hall Signal input 1
    HALL2             : in  std_logic;  -- Hall Signal input 2
    HALL3             : in  std_logic); -- Hall Signal input 3
end tmc603_commutator;

architecture structure of tmc603_commutator is

  signal scf_clk_cnt_ff : std_logic_vector (3 downto 0);  -- SCF clock divider counter
  signal scf_clk_ff     : std_logic;    -- SCF clock FF

  signal DualChop_sg, ChopHS_sg : std_logic;  -- hallFX requires DualChop='1' and ChopHS='0'

  signal tmc603_clr_err_ff : std_logic;
  signal tmc603_enable_ff  : std_logic;

  component commutator is
    port (
      nRST       : in  std_logic; 
      CLK        : in  std_logic;  -- clock
      HallFX_EN  : in  std_logic;  -- HallFX(TM) enable select, '0' : H321 / '1' : E312           
      H3, H2, H1 : in  std_logic;  -- Hall sensor inputs
      E3, E2, E1 : in  std_logic;  -- Hall sensor emulation signal (HallFX) inputs
      DualChop   : in  std_logic;  -- '0' : chopper @ HS xor LS / '1':chopper @ HS and LS
      ChopHS     : in  std_logic;  -- DualChop='1' => ChopHS='0' : chopper @ LS / ChopHS='1' : chopper @ HS            
      DIR        : in  std_logic;  -- select direction of revolution                 
      BRAKE      : in  std_logic;  -- active brake (with PWM control)               
      OVC        : in  std_logic;  -- over current ('1':disable all bridges
      PWM        : in  std_logic;  -- PWM input (from uC) to adjust speed            
      TACHO      : out std_logic;  -- tacho signal output (H3 xor H2 xor H1 resp. E3 xor E2 xor E1 )
      DIR_COM    : out std_logic;  -- measured commutation direction output (check for Hall / HallFX)             
      W_H, W_L   : out std_logic;       -- HS, LS outputs for haltf bridger W
      V_H, V_L   : out std_logic;       -- HS, LS outputs for haltf bridger W
      U_H, U_L   : out std_logic);      -- HS, LS outputs for haltf bridger W
  end component commutator;

begin

  with hallFX_EN select DualChop_sg <= '1' when '1', DualChop when others;
  with hallFX_EN select ChopHS_sg   <= '0' when '1', ChopHS when others;

  commutator_i : commutator
    port map (
      nRST      => nRST,
      CLK       => clk,
      HallFX_EN => HallFX_EN,
      H3        => HALL3,
      H2        => HALL2,
      H1        => HALL1,
      E3        => TMC603_H3,
      E2        => TMC603_H2,
      E1        => TMC603_H1,
      DualChop  => DualChop,
      ChopHS    => ChopHS,
      DIR       => DIR,
      BRAKE     => BRAKE,
      OVC       => OVC,
      PWM       => PWM,
      TACHO     => TACHO,
      DIR_COM   => DIR_COM,
      U_H       => TMC603_BH1,
      U_L       => TMC603_BL1,
      V_H       => TMC603_BH2,
      V_L       => TMC603_BL2,
      W_H       => TMC603_BH3,
      W_L       => TMC603_BL3);

  scfclk_p :         -- SCF clock divider (pls. refer TMC603 datasheet 
  process(nRST, CLK)  -- concerning selection of SCF clock frequency)
  begin
    if nRST = '0' then
      scf_clk_cnt_ff <= (others => '0');
      scf_clk_ff     <= '1';
    elsif CLK'event and CLK = '1' then
      if (scf_clk_cnt_ff < "1000") then
        scf_clk_ff <= '0';
      else
        scf_clk_ff <= '1';
      end if;
      scf_clk_cnt_ff <= scf_clk_cnt_ff + 1;
    end if;
  end process;

  TMC603_SCCLK <= scf_clk_ff;

  reg_p :
  process(nRST, CLK)
  begin
    if nRST = '0' then
      tmc603_clr_err_ff <= '0';
      tmc603_enable_ff  <= '0';
    elsif (CLK'event and CLK = '1') then
      tmc603_enable_ff  <= TMC603_ERROR_OUT;
      -- do error handling ...
      tmc603_clr_err_ff <= '1';  --... and e.g. clear error flag of TMC603
    end if;
  end process;

  TMC603_CLR_ERR    <= tmc603_clr_err_ff;
  TMC603_ENABLE     <= tmc603_enable_ff;
  TMC603_SAMPLE_123 <= '1';     -- to be changed if used
  TMC603_SENSE_HI   <= '0';
  TMC603_BBM_EN     <= '1';

end structure;
